/*
 * chcavity.h
 *
 * RF cavity device server header file
 *
 * LP - 13.02.2003
 */

#ifndef CHCAVITY_H
#define CHCAVITY_H

#define N_AM    7   /* number of AM */
#define N_LOOPS 0

typedef char devname_t[20];       /* type for device name */

/* AD16 channel structure definition */
typedef struct ad16ch_s {
  u_int chan;   /* channel number */
  double a,     /* calibration parameters: ax^3 + bx^2 + cx + d */
         b,
         c,
         d,
         min,   /* min value */
         max;   /* max value */
} ad16ch_t;   /* AD16 channel type */

#define N_AD16_CHANS  4

/* structure for AD16 board addressing */
typedef struct ad16_s {
  devname_t devname;
  ad16ch_t vlt,      /* cavity voltage */
           inpwr,    /* input power */
           rflpwr,   /* reflected power */
           posit;    /* mech. tuno position */
} ad16_t;

/* ADTH16 channel structure definition */
typedef ad16ch_t adth16ch_t;

#define N_ADTH16_CHANS  16

/* structure for ADTH16 board addressing */
typedef struct adth16_s {
  devname_t devname;
  adth16ch_t temp[N_ADTH16_CHANS];  /* all 16 channels used! */
} adth16_t;

/* structures for DIGIO board addressing */
typedef struct digio1_s {
  devname_t devname;
  u_short in_mask,       /* input mask */
          vacint_mask;   /* vacuum interlock */
} digio1_t;
  
typedef struct digio2_s {
  devname_t devname;
  u_short in_mask,       /* input mask */
          posit_mask;    /* tuno out of range */
} digio2_t;

#define VACINT_MASK 0x0001  /* vacuum interlock */
#define POSIT_MASK  0x0002  /* tuno out of range */

/*
 * resource struct definition
 */
typedef struct {
  ad16_t ad16;        /* one AD16 */
  adth16_t adth16;    /* one ADTH16 */
  digio1_t digio1;    /* two DIGIO */
  digio2_t digio2;
} resource_type;

#endif  /* CHCAVITY_H */

/* EOF */
