/*
 * cmcavity.c
 *
 * RF cavity device server
 *
 * LP - 13.02.2003
 * LP - 21.11.2008 - input voltage values > 0
 */

/*
 * system includes
 */
#include <setsys.h>
#include <stdio.h>
#include <math.h>    /* for fabs() */
#include <module.h>
#include <modes.h>
#include <stdlib.h>  /* for atoi() */
#include <string.h>  /* for strcmp() */
#include <errno.h>
#include <types.h>

/*
 * elettra includes
 */
#include <cherrs.h>   /* error numbers */
#include <cherrt.h>   /* error types */
#include "chlogs.h"   /* logging funcs */

/*
 * felix includes
 */
#include "chdevsrv.h"  /* generic device server stuff */
#include "chgetres.h"  /* resource reading stuff */

/*
 * driver includes
 */
#include "chad16.h"
#include "chdigio.h"

/*
 * device server includes
 */
#include "chcavity.h"

/*
 * global vars
 */
/* some resource stuff statically initialized */
resource_type resources;

/*
 * vars from generic part
 */
extern char loc_msg[200];
extern srvmod_type *srv_mod;  /* pointer to server module data structure */
extern loop_type *loop_var;
extern call_type *call_pnt;
extern str_typ res_err;       /* resource reading global error string */

/*
 * function prototypes
 */
int read_resources(char *);  /* routine di lettura delle risorse */
void felix_terminat(int);    /* specific part resources deallocation */
void iniz_hw(void);          /* HW initializations */
void go_on_loop(int);       /* server loops stuff */

/* server routines */
void vlt_rr(void);
void inpwr_rr(void);
void rflpwr_rr(void);
void posit_rr(void);
void temp_rr(void);
void temp_rm(void);
void stat_rf(void);


/*
 * functions prototypes from generic part of
 * device server, csdevsrv.c
 */
/* see chdevsrv.h */

/* AM -> device server routine association table */
act_mod_typ amtab[] = {
  "VLT",    "RR",   vlt_rr,     /* 1 */
  "INPWR",  "RR",   inpwr_rr,   /* 2 */
  "RFLPWR", "RR",   rflpwr_rr,  /* 3 */
  "POSIT",  "RR",   posit_rr,   /* 4 */
  "TEMP",   "RR",   temp_rr,    /* 5 */
  "TEMP",   "RM",   temp_rm,    /* 6 */
  "STAT",   "RF",   stat_rf     /* 7 */
};


/*
 * read_resources()
 * open and read the resources file
 */
int read_resources(char *filename)
{
  int i;
  char buf[20];
  
  if (read_res(filename) < 0) {
    printf("%s: cannot read resource file %s, error %d\n", \
           gettime(), filename, errno);
    return(-1);  /* some file handling error? */
  }
  /*
   * AD16 stuff
   */
  if (get_res(resources.ad16.devname, "ad16.devname", STRING) || \
      get_res(&resources.ad16.vlt.chan, "ad16.vlt.chan", INTEGER) || \
      get_res(&resources.ad16.vlt.a, "ad16.vlt.a", DOUBLE) || \
      get_res(&resources.ad16.vlt.b, "ad16.vlt.b", DOUBLE) || \
      get_res(&resources.ad16.vlt.c, "ad16.vlt.c", DOUBLE) || \
      get_res(&resources.ad16.vlt.d, "ad16.vlt.d", DOUBLE) || \
      get_res(&resources.ad16.vlt.min, "ad16.vlt.min", DOUBLE) || \
      get_res(&resources.ad16.vlt.max, "ad16.vlt.max", DOUBLE) || \
      get_res(&resources.ad16.inpwr.chan, "ad16.inpwr.chan", INTEGER) || \
      get_res(&resources.ad16.inpwr.a, "ad16.inpwr.a", DOUBLE) || \
      get_res(&resources.ad16.inpwr.b, "ad16.inpwr.b", DOUBLE) || \
      get_res(&resources.ad16.inpwr.c, "ad16.inpwr.c", DOUBLE) || \
      get_res(&resources.ad16.inpwr.d, "ad16.inpwr.d", DOUBLE) || \
      get_res(&resources.ad16.inpwr.min, "ad16.inpwr.min", DOUBLE) || \
      get_res(&resources.ad16.inpwr.max, "ad16.inpwr.max", DOUBLE) || \
      get_res(&resources.ad16.rflpwr.chan, "ad16.rflpwr.chan", INTEGER) || \
      get_res(&resources.ad16.rflpwr.a, "ad16.rflpwr.a", DOUBLE) || \
      get_res(&resources.ad16.rflpwr.b, "ad16.rflpwr.b", DOUBLE) || \
      get_res(&resources.ad16.rflpwr.c, "ad16.rflpwr.c", DOUBLE) || \
      get_res(&resources.ad16.rflpwr.d, "ad16.rflpwr.d", DOUBLE) || \
      get_res(&resources.ad16.rflpwr.min, "ad16.rflpwr.min", DOUBLE) || \
      get_res(&resources.ad16.rflpwr.max, "ad16.rflpwr.max", DOUBLE) || \
      get_res(&resources.ad16.posit.chan, "ad16.posit.chan", INTEGER) || \
      get_res(&resources.ad16.posit.a, "ad16.posit.a", DOUBLE) || \
      get_res(&resources.ad16.posit.b, "ad16.posit.b", DOUBLE) || \
      get_res(&resources.ad16.posit.c, "ad16.posit.c", DOUBLE) || \
      get_res(&resources.ad16.posit.d, "ad16.posit.d", DOUBLE) || \
      get_res(&resources.ad16.posit.min, "ad16.posit.min", DOUBLE) || \
      get_res(&resources.ad16.posit.max, "ad16.posit.max", DOUBLE)) {
    return -1;
  }
  /*
   * ADTH16 stuff
   */
  if (get_res(resources.adth16.devname, "adth16.devname", STRING))
    return -1;
  for (i = 0; i < N_ADTH16_CHANS; i++) {
    if (get_res(&resources.adth16.temp[i].chan, \
                sprn(buf, "adth16.temp[%d].chan", i), INTEGER) || \
        get_res(&resources.adth16.temp[i].a, \
                sprn(buf, "adth16.temp[%d].a", i), DOUBLE) || \
        get_res(&resources.adth16.temp[i].b, \
                sprn(buf, "adth16.temp[%d].b", i), DOUBLE) || \
        get_res(&resources.adth16.temp[i].c, \
                sprn(buf, "adth16.temp[%d].c", i), DOUBLE) || \
        get_res(&resources.adth16.temp[i].d, \
                sprn(buf, "adth16.temp[%d].d", i), DOUBLE) || \
        get_res(&resources.adth16.temp[i].min, \
                sprn(buf, "adth16.temp[%d].min", i), DOUBLE) || \
        get_res(&resources.adth16.temp[i].max, \
                sprn(buf, "adth16.temp[%d].max", i), DOUBLE)) {
      return -1;
    }
  }
  /*
   * DIGIO stuff
   */
  if (get_res(resources.digio1.devname, "digio1.devname", STRING) || \
      get_res(&resources.digio1.in_mask, "digio1.in_mask", USHORT) || \
      get_res(&resources.digio1.vacint_mask, "digio1.vacint_mask", USHORT)) {
    return -1;
  }
  if (get_res(resources.digio2.devname, "digio2.devname", STRING) || \
      get_res(&resources.digio2.in_mask, "digio2.in_mask", USHORT) || \
      get_res(&resources.digio2.posit_mask, "digio2.posit_mask", USHORT)) {
    return -1;
  }
  
#ifdef DEBUGRES
  fprintf(stderr, "ad16.devname = %s\n", resources.ad16.devname);
  fprintf(stderr, "ad16.vlt.chan = %d\n", resources.ad16.vlt.chan);
  fprintf(stderr, "ad16.vlt.a = %e\n", resources.ad16.vlt.a);
  fprintf(stderr, "ad16.vlt.b = %e\n", resources.ad16.vlt.b);
  fprintf(stderr, "ad16.vlt.c = %e\n", resources.ad16.vlt.c);
  fprintf(stderr, "ad16.vlt.d = %e\n", resources.ad16.vlt.d);
  fprintf(stderr, "ad16.vlt.min = %e\n", resources.ad16.vlt.min);
  fprintf(stderr, "ad16.vlt.max = %e\n", resources.ad16.vlt.max);
  fprintf(stderr, "ad16.inpwr.chan = %d\n", resources.ad16.inpwr.chan);
  fprintf(stderr, "ad16.inpwr.a = %e\n", resources.ad16.inpwr.a);
  fprintf(stderr, "ad16.inpwr.b = %e\n", resources.ad16.inpwr.b);
  fprintf(stderr, "ad16.inpwr.c = %e\n", resources.ad16.inpwr.c);
  fprintf(stderr, "ad16.inpwr.d = %e\n", resources.ad16.inpwr.d);
  fprintf(stderr, "ad16.inpwr.min = %e\n", resources.ad16.inpwr.min);
  fprintf(stderr, "ad16.inpwr.max = %e\n", resources.ad16.inpwr.max);
  fprintf(stderr, "ad16.rflpwr.chan = %d\n", resources.ad16.rflpwr.chan);
  fprintf(stderr, "ad16.rflpwr.a = %e\n", resources.ad16.rflpwr.a);
  fprintf(stderr, "ad16.rflpwr.b = %e\n", resources.ad16.rflpwr.b);
  fprintf(stderr, "ad16.rflpwr.c = %e\n", resources.ad16.rflpwr.c);
  fprintf(stderr, "ad16.rflpwr.d = %e\n", resources.ad16.rflpwr.d);
  fprintf(stderr, "ad16.rflpwr.min = %e\n", resources.ad16.rflpwr.min);
  fprintf(stderr, "ad16.rflpwr.max = %e\n", resources.ad16.rflpwr.max);
  fprintf(stderr, "ad16.posit.chan = %d\n", resources.ad16.posit.chan);
  fprintf(stderr, "ad16.posit.a = %e\n", resources.ad16.posit.a);
  fprintf(stderr, "ad16.posit.b = %e\n", resources.ad16.posit.b);
  fprintf(stderr, "ad16.posit.c = %e\n", resources.ad16.posit.c);
  fprintf(stderr, "ad16.posit.d = %e\n", resources.ad16.posit.d);
  fprintf(stderr, "ad16.posit.min = %e\n", resources.ad16.posit.min);
  fprintf(stderr, "ad16.posit.max = %e\n", resources.ad16.posit.max);
  
  fprintf(stderr, "adth16.devname = %s\n", resources.adth16.devname);
  for (i = 0; i < N_ADTH16_CHANS; i++) {
    fprintf(stderr, "adth16.temp[%d].chan = %d\n", \
            i, resources.adth16.temp[i].chan);
    fprintf(stderr, "adth16.temp[%d].a = %e\n", \
            i, resources.adth16.temp[i].a);
    fprintf(stderr, "adth16.temp[%d].b = %e\n", \
            i, resources.adth16.temp[i].b);
    fprintf(stderr, "adth16.temp[%d].c = %e\n", \
            i, resources.adth16.temp[i].c);
    fprintf(stderr, "adth16.temp[%d].d = %e\n", \
            i, resources.adth16.temp[i].d);
    fprintf(stderr, "adth16.temp[%d].min = %e\n", \
            i, resources.adth16.temp[i].min);
    fprintf(stderr, "adth16.temp[%d].max = %e\n", \
            i, resources.adth16.temp[i].max);
  }
  
  fprintf(stderr, "digio1.devname = %s\n", resources.digio1.devname);
  fprintf(stderr, "digio1.in_mask = 0x%04x\n", resources.digio1.in_mask);
  fprintf(stderr, "digio1.vacint_mask = 0x%04x\n", \
          resources.digio1.vacint_mask);
  
  fprintf(stderr, "digio2.devname = %s\n", resources.digio2.devname);
  fprintf(stderr, "digio2.in_mask = 0x%04x\n", resources.digio2.in_mask);
  fprintf(stderr, "digio2.posit_mask = 0x%04x\n", \
          resources.digio2.posit_mask);
#endif
  free_res();  /* free memory */
  return 0;
}  /* end read_resources() */


/*
 * felix_terminat()
 * specific part resources deallocation
 */
void felix_terminat(int sig)
{
  return;
}   /* end felix_teminat() */


/*
 * iniz_hw()
 * hardware inizialization
 */
void iniz_hw(void)
{
  ad16_ioctl_t adth16;
  
  /* initialize driver for ADTH16 */
  adth16.channel = 0;
  adth16.data[0] = ADTH16;
  adth16.data[1] = 0;
  if (equip(resources.adth16.devname, AD16_WMODEL, (char *)&adth16)) {
    printf("%s: iniz_hw(): device = %s, ioctl = %d, error = %d\n", \
           gettime(), resources.adth16.devname, AD16_WMODEL, errno);
    die(errno);
  }
  
  /* initialize DIGIOs */
  if (equip(resources.digio1.devname, DIGIO_DIGIT, \
            (char *)&resources.digio1.in_mask)) {
    printf("%s: iniz_hw(): device = %s, ioctl = %d, error = %d\n", \
           gettime(), resources.digio1.devname, DIGIO_DIGIT, errno);
    die(errno);
  }
  if (equip(resources.digio2.devname, DIGIO_DIGIT, \
            (char *)&resources.digio2.in_mask)) {
    printf("%s: iniz_hw(): device = %s, ioctl = %d, error = %d\n", \
           gettime(), resources.digio2.devname, DIGIO_DIGIT, errno);
    die(errno);
  }
  
  return;
}  /* end iniz_hw() */


/*
 * vlt_rr()
 */
void vlt_rr(void)
{
  ad16_ioctl_t ad16;
  double vlt;
  
  ad16.channel = resources.ad16.vlt.chan;
  ad16.data[0] = 0;
  ad16.data[1] = 0;
  if (equip(resources.ad16.devname, AD16_RADC12, (char *)&ad16)) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Ioctl error:", resources.ad16.devname, AD16_RADC12);
    return;
  }
  /* calculate voltage value with calibration parameters */
  vlt = ad16.data[0];
  if (vlt < 0)
    vlt = 0.0;
  vlt = resources.ad16.vlt.a * vlt * vlt * vlt + \
        resources.ad16.vlt.b * vlt * vlt + \
        resources.ad16.vlt.c * vlt + \
        resources.ad16.vlt.d;
  *((double *)call_pnt->data) = vlt;
  call_pnt->bytecount = sizeof(vlt);
  
#ifdef DEBUGCALLS
  fprintf(stderr, "vlt_rr(): data = %e, bytecount = %d\n", \
          *((double *)call_pnt->data), call_pnt->bytecount);
  fflush(stderr);
#endif
}  /* end vlt_rr() */


/*
 * inpwr_rr()
 */
void inpwr_rr(void)
{
  ad16_ioctl_t ad16;
  double inpwr;
  
  ad16.channel = resources.ad16.inpwr.chan;
  ad16.data[0] = 0;
  ad16.data[1] = 0;
  if (equip(resources.ad16.devname, AD16_RADC12, (char *)&ad16)) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Ioctl error:", resources.ad16.devname, AD16_RADC12);
    return;
  }
  /* calculate voltage value with calibration parameters */
  inpwr = ad16.data[0];
  inpwr = resources.ad16.inpwr.a * inpwr * inpwr * inpwr + \
          resources.ad16.inpwr.b * inpwr * inpwr + \
          resources.ad16.inpwr.c * inpwr + \
          resources.ad16.inpwr.d;
  *((double *)call_pnt->data) = inpwr;
  call_pnt->bytecount = sizeof(inpwr);
  
#ifdef DEBUGCALLS
  fprintf(stderr, "inpwr_rr(): data = %e, bytecount = %d\n", \
          *((double *)call_pnt->data), call_pnt->bytecount);
  fflush(stderr);
#endif
}  /* end inpwr_rr() */


/*
 * rflpwr_rr()
 */
void rflpwr_rr(void)
{
  ad16_ioctl_t ad16;
  double rflpwr;
  
  ad16.channel = resources.ad16.rflpwr.chan;
  ad16.data[0] = 0;
  ad16.data[1] = 0;
  if (equip(resources.ad16.devname, AD16_RADC12, (char *)&ad16)) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Ioctl error:", resources.ad16.devname, AD16_RADC12);
    return;
  }
  /* calculate voltage value with calibration parameters */
  rflpwr = ad16.data[0];
  rflpwr = resources.ad16.rflpwr.a * rflpwr * rflpwr * rflpwr + \
           resources.ad16.rflpwr.b * rflpwr * rflpwr + \
           resources.ad16.rflpwr.c * rflpwr + \
           resources.ad16.rflpwr.d;
  *((double *)call_pnt->data) = rflpwr;
  call_pnt->bytecount = sizeof(rflpwr);
  
#ifdef DEBUGCALLS
  fprintf(stderr, "rflpwr_rr(): data = %e, bytecount = %d\n", \
          *((double *)call_pnt->data), call_pnt->bytecount);
  fflush(stderr);
#endif
}  /* end rflpwr_rr() */


/*
 * posit_rr()
 */
void posit_rr(void)
{
  ad16_ioctl_t ad16;
  double posit;
  
  ad16.channel = resources.ad16.posit.chan;
  ad16.data[0] = 0;
  ad16.data[1] = 0;
  if (equip(resources.ad16.devname, AD16_RADC12, (char *)&ad16)) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Ioctl error:", resources.ad16.devname, AD16_RADC12);
    return;
  }
  /* calculate voltage value with calibration parameters */
  posit = ad16.data[0];
  posit = resources.ad16.posit.a * posit * posit * posit + \
          resources.ad16.posit.b * posit * posit + \
          resources.ad16.posit.c * posit + \
          resources.ad16.posit.d;
  *((double *)call_pnt->data) = posit;
  call_pnt->bytecount = sizeof(posit);
  
#ifdef DEBUGCALLS
  fprintf(stderr, "posit_rr(): data = %e, bytecount = %d\n", \
          *((double *)call_pnt->data), call_pnt->bytecount);
  fflush(stderr);
#endif
}  /* end posit_rr() */


/*
 * temp_rr()
 */
void temp_rr(void)
{
  ad16_ioctl_t ad16;
  double *temp, tmp;
  int i;
  
  temp = (double *)call_pnt->data;
  
  for (i = 0; i < N_ADTH16_CHANS; i++) {
    ad16.channel = resources.adth16.temp[i].chan;
    ad16.data[0] = 0;
    ad16.data[1] = 0;
    if (equip(resources.adth16.devname, AD16_RADCC16, (char *)&ad16)) {
      set_err(call_pnt, errno, ETERROR + ETAPPL, \
              "Ioctl error:", resources.ad16.devname, AD16_RADCC16);
      return;
    }
    tmp = (double)ad16.data[0];
    /*
     * calibration
     */
    temp[i] = resources.adth16.temp[i].a * tmp * tmp * tmp + \
              resources.adth16.temp[i].b * tmp * tmp + \
              resources.adth16.temp[i].c * tmp + \
              resources.adth16.temp[i].d;
  }
  call_pnt->bytecount = N_ADTH16_CHANS * sizeof(*temp);
  
#ifdef DEBUGCALLS
  fprintf(stderr, "temp_rr(): \n");
  for (i = 0; i < N_ADTH16_CHANS; i++)
    fprintf(stderr, "  data[%d] = %e\n", i, temp[i]);
  fprintf(stderr, "  bytecount = %d\n", call_pnt->bytecount);
  fflush(stderr);
#endif
}  /* end temp_rr() */


/*
 * temp_rm()
 */
void temp_rm(void)
{
  char *c_data;
  int i;

  c_data = (char *)call_pnt->data;
  
  for (i = 0; i < N_ADTH16_CHANS; i++)
    c_data += sprintf(c_data, "TEMP%d_RR,", resources.adth16.temp[i].chan);
  *(c_data - 1) = '\0';   /* clear last , */
  call_pnt->bytecount = strlen((char *)call_pnt->data);
  
#ifdef DEBUGCALLS
  fprintf(stderr, "temp_rm():\n");
  fprintf(stderr, "  data = %s\n", (char *)call_pnt->data);
  fprintf(stderr, "  bytecount = %d\n", call_pnt->bytecount);
  fflush(stderr);
#endif
}  /* end temp_rm() */


/*
 * stat_rf()
 */
void stat_rf(void)
{
  u_short sdata;
  u_int status = 0;
  
  if (equip(resources.digio1.devname, DIGIO_RDDIGIN, (char *)&sdata)) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Ioctl error:", resources.digio1.devname, DIGIO_RDDIGIN);
    return;
  }
  if (sdata & resources.digio1.vacint_mask)
    status |= VACINT_MASK;
  
  if (equip(resources.digio2.devname, DIGIO_RDDIGIN, (char *)&sdata)) {
    set_err(call_pnt, errno, ETERROR + ETAPPL, \
            "Ioctl error:", resources.digio2.devname, DIGIO_RDDIGIN);
    return;
  }
  if (sdata & resources.digio2.posit_mask)
    status |= POSIT_MASK;
  
  *((u_int *)call_pnt->data) = status;
  call_pnt->bytecount = sizeof(status);
  
#ifdef DEBUGCALLS
  fprintf(stderr, "stat_rf(): data = 0x%04x, bytecount = %d\n",
          *((u_int *)call_pnt->data), call_pnt->bytecount);
  fflush(stderr);
#endif
}  /* end stat_rf() */


/*
 * go_on_loop()
 * discriminate loop and call proper function
 */
void go_on_loop(int loop_num)
{
  switch (loop_num) {
    default:
      printf("%s: loop %d not implemented!", gettime(), loop_num);
  }  /* end switch */
}  /* end go_on_loop() */


/*
 * main()
 */
int main(int argc, char *argv[])
{
  int chnum;
  
  /* check input parameters */
  if (argc < 3) {
    printf("Usage: %s <channel number> <resource file>\n", argv[0]);
    exit(0);
  }

  /* generic part initialization */
  iniz(argv[1], N_LOOPS, amtab, N_AM);

  /* read resources from resource file */
  if (read_resources(argv[2]) < 0) {
    printf("%s: %s: %s\n", gettime(), argv[0], res_err);
    free_res();
    die(0);
  }

  iniz_hw();

  server_loop();  /* server main loop */
  
}  /* end main() */

/* EOF */
